#!/usr/bin/env python
# ScraperWiki Limited
# Ian Hopkinson, 2013-06-21
# -*- coding: utf-8 -*-


#import sys
#import codecs
# sys.stdout = codecs.getwriter('utf-8')(sys.stdout)
"""
Tell us what this does
"""

import os
from .pdftables import get_pdf_page, page_to_tables
from os.path import join, dirname
from . import pdftables_analysis as pta
from .display import to_string, get_dimensions
from io import StringIO


PDF_TEST_FILES = os.path.join(os.pardir, 'fixtures\sample_data')
#PDF_TEST_FILES = 'fixtures\sample_data'
hints = []
# SelectedPDF = "cit0613.pdf"
# pagenumber = 1 # Wierd - each word is a figure?. First page can be broken
# down to boxes containing no text 
#SelectedPDF = "COPAWEEKLYJUNE52013.pdf" #1
#pagenumber = 1 # parses if we just accept the whole page as a table, extend_y = True
#SelectedPDF = "COPAMONTHLYMay2013.pdf"
#pagenumber = 1 # Fails because it's two column and there are two tables. May be fixed by local / adaptive minima
# SelectedPDF = "13_06_12_10_36_58_boletim_ingles_junho_2013.pdf" # 32 pages, Long document with many tables
#pagenumber = 5 # No tables on this page
#pagenumber = 6
# pagenumber = 8 #good mix of text and table
#SelectedPDF = "1359397366Final_Coceral grain estimate_2012_December.pdf" #3 pages Slow to interpret
#pagenumber = 1 # couple of places where columns are welded, fixed with going to single characters
#pagenumber = 2 # works without going to single characters
#pagenumber = 3 # As for page 1 - some columns are welded, fixed with going to single characters
#SelectedPDF = "ClinicalResearchDisclosureReport2012Q2.pdf"
# pagenumber = 1 # PDFEncryptionError('Unknown algorithm: param=%r' % param)
# SelectedPDF = "argentina_diputados_voting_record.pdf" # 1page
# pagenumber = 1
# hints = ['Apellido', '']
#SelectedPDF = "bo_page24.pdf" # 1 page Multi-column text and tables mixed on the page
#pagenumber = 1 # Fails - need a multicolumn detector
#SelectedPDF = "tabla_subsidios.pdf" # 1 page, fairly simple table
#pagenumber = 1 # works fine
# SelectedPDF = "m27-dexpeg2-polymer.pdf" # No tables, two column text
# SelectedPDF = "m29-JDent36s2-7.pdf" # One table, two column text, equations may cause problems?
# pagenumber = 4 # Works OK but some - signs are missed and we need to use extend_y
#SelectedPDF = "m30-JDent36s15-20.pdf" # several tables
#pagenumber = 4 # fails two different tables on one page, two column layout but tables span page -
#pagenumber = 5 # fails, very small table in two column layout, yComb is not ordered
# Open a PDF file.

SelectedPDF = "2012.01.PosRpt.pdf" # 7 pages works fine in pdfminer, 4 for first test 2012.01.PosRpt.pdf
pagenumber = 1 # Table too small to find - needs hints
hints = ["% Change", "Uncommited"]
#pagenumber = 2 # Looks really nice
#pagenumber = 3 # Looks really nice
#pagenumber = 4 # the original!
#pagenumber = 5 # easier
#pagenumber = 6 # Columns welded: 21.94% 157, 201, 476 and where there is a spanning footer, cells overwriting
#pagenumber = 7 # welding caused by isolated spanning text (fixed by dynamic threshold)

# This needs "Get table from hints", really heavy hints!
#fh = pdftables.filehandleFromURL("http://www.candyusa.com/files/1st%20qtr%202013%20report.pdf")
#pagenumber = 1

#SelectedPDF = "pdf_prc_prod_1_7_1288_acucar-vhp-vendido-mercado-externo_sao-paulo_mensal.pdf"
#pagenumber = 1

#SelectedPDF = "commodity-prices_en.pdf"
#pagenumber = 1

SelectedPDF = "AnimalExampleTables.pdf" # 7 pages works fine in pdfminer, 4 for first test 2012.01.PosRpt.pdf
pagenumber = 2

filepath = os.path.join(PDF_TEST_FILES, SelectedPDF)
fh = open(filepath, 'rb')
#pta.plotAllPages(fh)



pdf_page = get_pdf_page(fh, pagenumber)

table, diagnosticData = page_to_tables(pdf_page, extend_y=False, hints=hints, atomise=False)

fig, ax1 = pta.plotpage(diagnosticData)

result = StringIO()
(columns, rows) = get_dimensions(table)
result.write("     {} columns, {} rows\n".format(columns, rows))

print(to_string(table))




# BoxList = plotAllPages(open(filepath, 'rb'))



#!/usr/bin/env python
# ScraperWiki Limited
# Ian Hopkinson, 2013-06-20
# -*- coding: utf-8 -*-


import sys
import codecs
sys.stdout = codecs.getwriter('utf-8')(sys.stdout)

"""
Analysis and visualisation library for pdftables
"""


from . import pdftables as pt
import matplotlib.pyplot as plt
from mpl_toolkits.axes_grid1 import make_axes_locatable
from .tree import Leaf, LeafList
from pdfminer.pdfpage import PDFPage


FilterOptions = ['LTPage','LTTextBoxHorizontal','LTFigure','LTLine','LTRect','LTImage','LTTextLineHorizontal','LTCurve', 'LTChar', 'LTAnon']
Colours       = ['black' ,'green'              ,'black'   ,'red'   ,'red'   ,'black'  ,'blue'                ,'red'    , 'red'   , 'White']

ColourTable = dict(list(zip(FilterOptions, Colours)))

LEFT = 0
TOP = 3
RIGHT = 2
BOTTOM = 1


def plotpage(d):
#def plotpage(BoxList,xhistleft,xhistright,yhisttop,yhistbottom,xComb,yComb):
#    global ColourTable
    """This is from pdftables"""
    #columnProjectionThreshold = threshold_above(columnProjection,columnThreshold)
    #colDispHeight = max(columnProjection.values())*0.8
    #columnProj = dict(zip(columnProjectionThreshold, [colDispHeight]*len(columnProjectionThreshold)))
    """End display only code"""

    """This is from pdftables"""
    #rowProjectionThreshold = threshold_above(rowProjection,rowThreshold)
    # rowDispHeight = max(rowProjection.values())*0.8
    # rowProj = dict(zip(rowProjectionThreshold, [rowDispHeight]*len(rowProjectionThreshold)))
    """End display only code"""

    fig = plt.figure()
    ax1 = fig.add_subplot(111)
    ax1.axis('equal')
    for boxstruct in d.box_list:
        box = boxstruct.bbox
        thiscolour = ColourTable[boxstruct.classname]
        ax1.plot([box[0],box[2],box[2],box[0],box[0]],[box[1],box[1],box[3],box[3],box[1]],color = thiscolour )

    # fig.suptitle(title, fontsize=15)
    divider = make_axes_locatable(ax1)
    #plt.setp(ax1.get_yticklabels(),visible=False)
    ax1.yaxis.set_label_position("right")

    if d.top_plot: 
        axHistx = divider.append_axes("top", 1.2, pad=0.1, sharex=ax1)
        axHistx.plot(list(map(float,list(d.top_plot.keys()))),list(map(float,list(d.top_plot.values()))), color = 'red')
        
    if d.left_plot:
        axHisty = divider.append_axes("left", 1.2, pad=0.1, sharey=ax1)
        axHisty.plot(list(map(float,list(d.left_plot.values()))),list(map(float,list(d.left_plot.keys()))), color = 'red')
    
    if d.y_comb:
        miny = min(d.y_comb)
        maxy = max(d.y_comb)

        for x in d.x_comb:
            ax1.plot([x,x],[miny,maxy],color = "black")
            axHistx.scatter(x,0,color = "black")

    if d.x_comb:
        minx = min(d.x_comb)
        maxx = max(d.x_comb)

        for y in d.y_comb:
            ax1.plot([minx,maxx],[y,y],color = "black")
            axHisty.scatter(1,y,color = "black")

    plt.draw()
    plt.show(block = False)

    return fig, ax1

def plothistogram(hist):
    fig = plt.figure()
    ax1 = fig.add_subplot(111)
    # ax1.axis('equal')
    ax1.scatter(list(map(float,list(hist.keys()))),list(map(float,list(hist.values()))))
    #fig.suptitle('%s : Page %d' % (SelectedPDF,pagenumber), fontsize=15)
    plt.draw()
    return fig

def plotAllPages(fh):
    #tol = 5 # This is the tolerance for histogram rounding
    
    fig_list = []
    ax1_list = []
    
    interpreter, device = pt.initialize_pdf_interpreter()
    pages = PDFPage.get_pages(fh)
    # print SelectedPDF
    #Creator = doc.info[0]['Creator']
    #print("Created by: %s" % Creator)
    #flt = 'LTTextLineHorizontal'
    #flt = ['LTPage','LTTextLineHorizontal']
    # flt = ['LTPage','LTFigure','LTLine','LTRect','LTImage','LTTextLineHorizontal','LTCurve']
    flt = ['LTPage','LTChar']
    for i,page in enumerate(pages):
        # page = next(doc.get_pages())

        interpreter.process_page(page)
    # receive the LTPage object for the page.
        layout = device.get_result()
        box_list = LeafList().populate(layout, interested = flt)

        ModalHeight = pt.calculate_modal_height(box_list)
        
        diagnostic_data = pt.TableDiagnosticData(
                box_list,
                {},
                {},
                [],
                [])
        
        fig, ax1 = plotpage(diagnostic_data)
        fig_list.append(fig)
        ax1_list.append(ax1)
        
        title = "page %d" % (i+1)
        fig.suptitle(title)
        #print "Page %d" % (i+1), ElementCount
        print(box_list.count())
        print("Modal character height: %d" % ModalHeight)

    return fig_list, ax1_list



#!/usr/bin/env python
# ScraperWiki Limited
# Ian Hopkinson, 2013-07-30
# -*- coding: utf-8 -*-

"""
Tests the Table class which contains metadata
"""
import sys
sys.path.append('code')

from pdftables import get_tables

from nose.tools import *

def test_it_includes_page_numbers():
    """
    page_number is 1-indexed, as defined in the PDF format
    table_number is 1-indexed
    """
    fh = open('fixtures/sample_data/AnimalExampleTables.pdf', 'rb')
    result = get_tables(fh)
    assert_equals(result[0].total_pages, 4)
    assert_equals(result[0].page_number, 2)
    assert_equals(result[1].total_pages, 4)
    assert_equals(result[1].page_number, 3)
    assert_equals(result[2].total_pages, 4)
    assert_equals(result[2].page_number, 4)

def test_it_includes_table_numbers():
    fh = open('fixtures/sample_data/AnimalExampleTables.pdf', 'rb')
    result = get_tables(fh)
    assert_equals(result[0].table_number_on_page, 1)
    assert_equals(result[0].total_tables_on_page, 1)




#!/usr/bin/env python
# ScraperWiki Limited
# Ian Hopkinson, 2013-06-17
# -*- coding: utf-8 -*-

"""
Finds tables tests
"""

import sys
sys.path.append('code')

import pdftables

from nose.tools import assert_equals

def test_atomise_does_not_disrupt_table_finding():
    fh = open('fixtures/sample_data/13_06_12_10_36_58_boletim_ingles_junho_2013.pdf', 'rb')
    pdf_page = pdftables.get_pdf_page(fh, 4)
    table1, _ = pdftables.page_to_tables(pdf_page, atomise=True, extend_y=False)
    table2, _ = pdftables.page_to_tables(pdf_page, atomise=False, extend_y=False)
    
    
    assert_equals(table1, table2)



#!/usr/bin/env python
# ScraperWiki Limited
# Ian Hopkinson, 2013-06-14
# -*- coding: utf-8 -*-

"""
Code to find tables in PDF files
"""

import os
# import requests
import scraperwiki # pdftoxml does not work on Windows
import lxml.html
import glob
import matplotlib.pyplot as plt
import collections
from .counter import Counter

# TODO - Use pdfminer
# TODO

def pdftoxml(filename,options):
    ConverterPath = str(r'C:\Users\Ian\BitBucketRepos\0939-AgraInforma\bin\pdftohtml.exe')
    directory = os.path.split(filename)[0]
    tmpxml = os.path.join(directory,"temph.xml")
    if tmpxml in os.listdir('.'):
        os.remove(tmpxml)
    cmd = '%s -xml %s "%s" %s' % (ConverterPath, options, filename, os.path.splitext(tmpxml)[0])

    os.system(cmd)

    f = open(tmpxml,'rb')
    content = f.read()
    f.close()

    return content

def processpage(page):
    left=[]
    width=[]
    top=[]
    right=[]
    for textchunk in (page is not None and page.xpath('text')):
        thisleft = int(textchunk.attrib.get('left'))
        thiswidth = int(textchunk.attrib.get('width'))
        left.append(thisleft)
        width.append(thiswidth)
        top.append(pageheight - int(textchunk.attrib.get('top')))
        right.append(thisleft + thiswidth)

    return pageheight,pagewidth,left,top,right

def plotpage(pageheight,pagewidth,pagenumber,SelectedPDF,left,top,right):
    fig = plt.figure()
    ax1 = fig.add_subplot(111)
    ax1.axis('equal')
    ax1.plot([0,pagewidth,pagewidth,0,0],[0,0,pageheight,pageheight,0])
    ax1.scatter(left, top, s=10, c='b', marker="s")
    ax1.scatter(right, top, s=10, c='r', marker="o")
    fig.suptitle('%s : Page %d' % (SelectedPDF,pagenumber), fontsize=15)
    plt.show()
    return fig

PDF_TEST_FILES = str(r'C:\Users\Ian\BitBucketRepos\0939-AgraInforma\fixtures')

# PDFList = glob.glob(os.path.join(PDF_TEST_FILES,'*.pdf'))

# SelectedPDF = 6 # 6 = cit0613.pdf - table is actually an image

# r = requests.get(os.path.join(PDF_TEST_FILES,PDFList[SelectedPDF]))
# options = ""
# xmldata = pdftoxml(os.path.join(PDF_TEST_FILES,PDFList[SelectedPDF]),options)

# xmldata = pdftoxml(os.path.join(PDF_TEST_FILES,"cit0613.pdf"),options) # Works but first page is an image
# xmldata = pdftoxml(os.path.join(PDF_TEST_FILES,"2012.01.PosRpt.pdf"),options) # PDF to HTML does not like
# xmldata = pdftoxml(os.path.join(PDF_TEST_FILES,"COPAWEEKLYJUNE52013.pdf"),options)
# xmldata = pdftoxml(os.path.join(PDF_TEST_FILES,"COPAMONTHLYMay2013.pdf"),options) # lxml doesn't like this one, interleaved <b> and <i> tags
# xmldata = pdftoxml(os.path.join(PDF_TEST_FILES,"13_06_12_10_36_58_boletim_ingles_junho_2013.pdf"),options) # Long document with many tables
# xmldata = pdftoxml(os.path.join(PDF_TEST_FILES,"1359397366Final_Coceral grain estimate_2012_December.pdf"),options)
# xmldata = pdftoxml(os.path.join(PDF_TEST_FILES,"ClinicalResearchDisclosureReport2012Q2.pdf"),options) # throws not allowed
# xmldata = pdftoxml(os.path.join(PDF_TEST_FILES,"argentina_diputados_voting_record.pdf"),options)
# xmldata = pdftoxml(os.path.join(PDF_TEST_FILES,"bo_page24.pdf"),options) # Multi-column text and tables mixed on the page
# xmldata = pdftoxml(os.path.join(PDF_TEST_FILES,"tabla_subsidios.pdf"),options) # Multi-column text and tables mixed on the page
SelectedPDF = "argentina_diputados_voting_record.pdf"

xmldata = pdftoxml(os.path.join(PDF_TEST_FILES,SelectedPDF),options)

root = lxml.etree.fromstring(xmldata)
pages = list(root)

# This is ok but


for page in pages:
    pagenumber = int(page.attrib.get("number"))
    pagewidth = int(page.attrib.get("width"))
    pageheight = int(page.attrib.get("height"))

    pageheight,pagewidth,left,top,right = processpage(page)

    fig = plotpage(pageheight,pagewidth,pagenumber,SelectedPDF,left,top,right)


    # counter=Counter(left)



#!/usr/bin/env python
# -*- coding: utf-8 -*-

# ScraperWiki Limited
# Ian Hopkinson, 2013-06-04


"""
Some experiments with pdfminer
http://www.unixuser.org/~euske/python/pdfminer/programming.html
Some help here:
http://denis.papathanasiou.org/2010/08/04/extracting-text-images-from-pdf-files
"""

# TODO Identify multi-column text, for multicolumn text detect per column
# TODO Dynamic / smarter thresholding
# TODO Handle argentina_diputados_voting_record.pdf automatically
# TODO Handle multiple tables on one page


import sys
import codecs

from pdfminer.pdfparser import PDFParser
from pdfminer.pdfpage import PDFPage
from pdfminer.pdfdocument import PDFDocument
from pdfminer.pdfinterp import PDFResourceManager, PDFPageInterpreter
from pdfminer.pdfdevice import PDFDevice
from pdfminer.layout import LAParams, LTPage
from pdfminer.converter import PDFPageAggregator

import collections

from .tree import Leaf, LeafList
import requests  # TODO: remove this dependency
from io import StringIO
import math
import numpy # TODO: remove this dependency
from .counter import Counter


def fround(x):
    return float(round(x))


IS_TABLE_COLUMN_COUNT_THRESHOLD = 3
IS_TABLE_ROW_COUNT_THRESHOLD = 3

class TableDiagnosticData(object):
    def __init__(self, box_list=LeafList(), top_plot=dict(), left_plot=dict(), x_comb=[], y_comb=[]):
        self.box_list = box_list
        self.top_plot = top_plot
        self.left_plot = left_plot
        self.x_comb = x_comb
        self.y_comb = y_comb

class Table(list):
    def __init__(self, content, page, page_total, table_index, table_index_total):
        super(Table, self).__init__(content)
        self.page_number = page
        self.total_pages = page_total
        self.table_number_on_page = table_index
        self.total_tables_on_page = table_index_total

LEFT = 0
TOP = 3
RIGHT = 2
BOTTOM = 1

def get_tables(fh):
    """
    Return a list of 'tables' from the given file handle, where a table is a
    list of rows, and a row is a list of strings.
    """
    result = []
    interpreter, device = initialize_pdf_interpreter()
    pages = list(PDFPage.get_pages(fh))
    doc_length = len(pages)
    for i, pdf_page in enumerate(pages):
        #print("Trying page {}".format(i + 1))
        if not page_contains_tables(pdf_page, interpreter, device):
            #print("Skipping page {}: no tables.".format(i + 1))
            continue

        # receive the LTPage object for the page.
        interpreter.process_page(pdf_page)
        processed_page = device.get_result()

        (table, _) = page_to_tables(
            processed_page,
            extend_y=True,
            hints=[],
            atomise=True)
        crop_table(table)
        result.append(Table(table,i+1,doc_length,1,1))

    return result


def crop_table(table):
    """
    Remove empty rows from the top and bottom of the table.
    """
    for row in list(table):  # top -> bottom
        if not any(cell.strip() for cell in row):
            table.remove(row)
        else:
            break

    for row in list(reversed(table)):  # bottom -> top
        if not any(cell.strip() for cell in row):
            table.remove(row)
        else:
            break


def initialize_pdf_interpreter():
    # Create a PDF resource manager object that stores shared resources.
    rsrcmgr = PDFResourceManager()
    # Create a PDF device object.
    device = PDFDevice(rsrcmgr)
    # Create a PDF interpreter object.
    interpreter = PDFPageInterpreter(rsrcmgr, device)
    # Process each page contained in the document.
    # for page in doc.get_pages():
    #    interpreter.process_page(page)

    # Set parameters for analysis.
    laparams = LAParams()
    laparams.word_margin = 0.0
    # Create a PDF page aggregator object.
    device = PDFPageAggregator(rsrcmgr, laparams=laparams)
    interpreter = PDFPageInterpreter(rsrcmgr, device)
    return interpreter, device


def contains_tables(fh):
    """
    contains_tables(fh) takes a file handle and returns a boolean array of the
    length of the document which is true for pages which contains tables
    """
    interpreter, device = initialize_pdf_interpreter()
    pages = PDFPage.get_pages(fh)

    return [page_contains_tables(p, interpreter, device) for
            p in pages]


def page_contains_tables(pdf_page, interpreter, device):
    # TODO: hide doc, interpreter, device inside a higher level Pdf class. It's
    # silly that we have to care about these (see function signature!!)

    interpreter.process_page(pdf_page)
    # receive the LTPage object for the page.
    layout = device.get_result()
    box_list = LeafList().populate(layout)
    for item in box_list:
        assert isinstance(item, Leaf), "NOT LEAF"
    yhist = box_list.histogram(Leaf._top).rounder(1)

    test = [k for k, v in list(yhist.items()) if v > IS_TABLE_COLUMN_COUNT_THRESHOLD]
    return len(test) > IS_TABLE_ROW_COUNT_THRESHOLD


def threshold_above(hist, threshold_value):
    """
    >>> threshold_above(Counter({518: 10, 520: 20, 530: 20, \
                                             525: 17}), 15)
    [520, 530, 525]
    """
    if not isinstance(hist, Counter):
        raise ValueError("requires Counter")  # TypeError then?

    above = [k for k, v in list(hist.items()) if v > threshold_value]
    return above


def comb(combarray, value):
    """
    Takes a sorted array and returns the interval number of the value passed to
    the function
    """
    # Raise an error in combarray not sorted
    if (combarray != sorted(combarray)) and (combarray != sorted(
            combarray, reverse=True)):
        raise Exception("comb: combarray is not sorted")

    index = -1
    if combarray[0] > combarray[-1]:
        for i in range(1, len(combarray)):
            if combarray[i - 1] >= value >= combarray[i]:
                index = i - 1
    else:
        for i in range(1, len(combarray)):
            if combarray[i - 1] <= value <= combarray[i]:
                index = i - 1

    return index


def apply_combs(box_list, x_comb, y_comb):
    """Allocates text to table cells using the x and y combs"""
    ncolumns = len(x_comb) - 1
    nrows = len(y_comb) - 1
    table_array = [[''] * ncolumns for j in range(nrows)]
    for box in box_list:
        y = fround(box.midline)
        x = fround(box.centreline)
        rowindex = comb(y_comb, y)
        columnindex = comb(x_comb, x)
        if rowindex != -1 and columnindex != -1:
            # there was already some content at this coordinate so we
            # concatenate (in an arbitrary order!)
            table_array[rowindex][columnindex] += box.text.rstrip('\n\r')

    return table_array


def comb_from_projection(projection, threshold, orientation):
    """Calculates the boundaries between cells from the projection of the boxes
    onto either the y axis (for rows) or the x-axis (for columns). These
    boundaries are known as the comb
    """
    if orientation=="row":
        tol=1
    elif orientation=="column":
        tol=3

    projection_threshold = threshold_above(projection, threshold)

    projection_threshold = sorted(projection_threshold)
    # need to generate a list of uppers (right or top edges)
    # and a list of lowers (left or bottom edges)

    # uppers = [k for k, v in yhisttop.items() if v > yThreshold]
    # lowers = [k for k, v in yhistbottom.items() if v > yThreshold]
    uppers = []
    lowers = []

    lowers.append(projection_threshold[0])
    for i in range(1, len(projection_threshold)):
        if projection_threshold[i] > (
                projection_threshold[i-1] + 1):
            uppers.append(projection_threshold[i - 1])
            lowers.append(projection_threshold[i])
    uppers.append(projection_threshold[-1])

    comb = comb_from_uppers_and_lowers(uppers, lowers, tol=tol,
                                       projection = projection)
    comb.reverse()

    return comb


def comb_from_uppers_and_lowers(uppers, lowers, tol=1, projection=dict()):
    """Called by comb_from_projection to calculate the comb given a set of
    uppers and lowers, which are upper and lower edges of the thresholded
    projection"""
    # tol is a tolerance to remove very small minima, increasing to 2 fowls up
    # row separation
    assert len(uppers) == len(lowers)
    uppers.sort(reverse=True)
    lowers.sort(reverse=True)
    comb = []
    comb.append(uppers[0])
    for i in range(1, len(uppers)):
        if (lowers[i - 1]-uppers[i])>tol:
            comb.append(find_minima(lowers[i - 1], uppers[i], projection))
            #comb.append(find_minima(lowers[i - 1], uppers[i]))

    comb.append(lowers[-1])

    return comb

def find_minima(lower, upper, projection=dict()):

    #print lower, upper, projection
    if len(projection)==0:
        idx = (lower + upper) / 2.0
    else:
        profile = []
        for i in range(upper, lower):
            #print projection[i]
            profile.append(projection[i])

        val, idx = min((val, idx) for (idx, val) in enumerate(profile))
        #val, idx = min(profile)
        idx = upper + idx

    return idx

def comb_extend(comb, minv, maxv):
    """Extend the comb to minv and maxv"""
    # TODO should this truncate if minv>minc or maxc>maxc
    # print y_comb
    # Find sort order of comb, convert to ascending
    reversed = False
    if comb[0] > comb[-1]:
        comb.reverse()
        reversed = True
    # Find min and max of comb
    minc = comb[0]
    maxc = comb[-1]
    # Get average row spacing
    rowSpacing = numpy.average(numpy.diff(comb))
    # Extend minimum
    if minv < minc:
        comb.reverse()
        comb.extend(list(numpy.arange(minc, minv, -rowSpacing))[1:])
        comb.reverse()
    # Extend maximum
    if maxv > maxc:
        comb.extend(list(numpy.arange(maxc, maxv, rowSpacing))[1:])

    if reversed:
        comb.reverse()
    return comb


def project_boxes(box_list, orientation, erosion=0):
    """
    Take a set of boxes and project their extent onto an axis
    """
    if orientation == "column":
        upper = RIGHT
        lower = LEFT
    elif orientation == "row":
        upper = TOP
        lower = BOTTOM

    projection = {}
    minv = fround(min([box.bbox[lower]
                 for box in box_list])) - 2  # ensure some overlap
    maxv = fround(max([box.bbox[upper] for box in box_list])) + 2

    # Initialise projection structure
    # print minv, maxv
    coords = list(range(int(minv), int(maxv)))
    projection = coords

    # print projection
    for box in box_list:
        for i in range(int(fround(box.bbox[lower])) + erosion,
                       int(fround(box.bbox[upper])) - erosion):
            # projection[i] += 1
            projection.append(i)

    return Counter(projection)


def get_pdf_page(fh, pagenumber):
    interpreter, device = initialize_pdf_interpreter()
    pages = list(PDFPage.get_pages(fh))

    try:
        page = pages[pagenumber - 1]
    except IndexError:
        raise IndexError("Invalid page number")

    interpreter.process_page(page)
    # receive the LTPage object for the page.
    processedPage = device.get_result()
    return processedPage

# def getTable(fh, page, extend_y=False, hints=[]):
#    """placeholder for tests, refactor out"""
#    return page_to_tables(get_pdf_page(fh, page), extend_y, hints)


def get_min_and_max_y_from_hints(box_list, top_string, bottom_string):
    miny = None
    maxy = None
    if top_string:
        top_box = [box for box in box_list if top_string in box.text]
        if top_box:
            maxy = top_box[0].top
    if bottom_string:
        bottomBox = [box for box in box_list if bottom_string in box.text]
        if bottomBox:
            miny = bottomBox[0].bottom
    return miny, maxy


def rounder(val, tol):
    """
    Utility function to round numbers to arbitrary tolerance
    """
    return fround((1.0 * val) / tol) * tol


#def filter_box_list_by_type(box_list, flt):
#    return [box for box in box_list if box.classname in flt]


def multi_column_detect(page):
    #TODO This function is under construction
    """
    Test for multiColumns from a box_list, returns an integer number of columns
    and a set of (left, right) pairs delineating any columns
    """
    # Ways to identify multicolumns:
    # 1. High fill factor compared to tables
    # 2. Gullies at textwidth/2, (textwidth/3, 2*textwidth/3)...
    # 3. Histogram of boxwidths with peak at some fraction of page width
    # This is like project_boxes but we are projecting the length of the
    # textbox onto the axis
    box_list = LeafList().populate(
        page, ['LTPage', 'LTTextLineHorizontal']).purge_empty_text()

    # Should use the LTPage object to get page bounding box
    box_list = filter_box_list_by_type(box_list, 'LTTextLineHorizontal')
    pile = {}
    vstep = 5  # should be scaled by modal row height
    minv = rounder(
        min([box.bottom for box in box_list]),
        5)  # ensure some overlap
    maxv = rounder(max([box.top for box in box_list]), 5)

    minx = fround(min([box.left for box in box_list]))  # ensure some overlap
    maxx = fround(max([box.right for box in box_list]))

    # Initialise projection structure
    # print minv, maxv

    coords = list(range(int(minv), int(maxv) + vstep, vstep))

    pile = collections.OrderedDict(list(zip(coords, [0] * len(coords))))
    # print projection
    for box in box_list:
        # print int(rounder(box.midline, 30)), box.width
        pile[int(rounder(box.midline, vstep))] += box.width

    for key, value in list(pile.items()):
        pile[key] = value // (maxx - minx)

    # Box width histogram
    bstep = 10
    boxhist = {}
    boxwidthmin = rounder(min([box.width for box in box_list]), bstep)
    boxwidthmax = rounder(max([box.width for box in box_list]), bstep)

    coords = list(range(int(boxwidthmin), int(boxwidthmax) + bstep, bstep))
    boxhist = collections.OrderedDict(list(zip(coords, [0] * len(coords))))
    for box in box_list:
        # print int(rounder(box.midline, 30)), box.width
        boxhist[int(rounder(box.width, bstep))] += 1

    nboxes = len(box_list)
    for key, value in list(boxhist.items()):
        boxhist[key] = float(value) / float(nboxes)
    # TODO: plt undefined
    fig = plt.figure()
    ax1 = fig.add_subplot(111)
    ax1.plot(list(map(float, list(boxhist.keys()))),
             list(map(float, list(boxhist.values()))), color='red')
    plt.show()

    # This is old fashion projection
    projection = project_boxes(box_list, 'column')
    # process key and value
    # print projection
    # projection = Counter(projection)
    # print projection
    return pile, projection


def page_to_tables(page, extend_y=False, hints=[], atomise=False):
    """
    Get a rectangular list of list of strings from one page of a document
    """
    if not isinstance(page, LTPage):
        raise TypeError("Page must be LTPage, not {}".format(page.__class__))

    table_array = []

    # For LTTextLine horizontal column and row thresholds of 3 work ok
    columnThreshold = 5  # 3 works for smaller tables
    rowThreshold = 3

    if atomise:
        flt = ['LTPage', 'LTTextLineHorizontal', 'LTChar']
    else:
        flt = ['LTPage', 'LTTextLineHorizontal']
    # flt = ['LTPage', 'LTTextLineHorizontal', 'LTFigure']
    box_list = LeafList().populate(page, flt).purge_empty_text()

    (minx, maxx, miny, maxy) = find_table_bounding_box(box_list, hints=hints)

    """If miny and maxy are None then we found no tables and should exit"""
    if miny is None and maxy is None:
       print("found no tables")
       return table_array, TableDiagnosticData()

    if atomise:
        box_list = box_list.filterByType(['LTPage', 'LTChar'])

    filtered_box_list = filter_box_list_by_position(
        box_list,
        miny,
        maxy,
        Leaf._midline)

    filtered_box_list = filter_box_list_by_position(
        filtered_box_list,
        minx,
        maxx,
        Leaf._centreline)

    # Project boxes onto horizontal axis
    column_projection = project_boxes(filtered_box_list, "column")

    # Project boxes onto vertical axis
    # Erode row height by a fraction of the modal text box height
    erodelevel = int(math.floor(calculate_modal_height(filtered_box_list) // 4))
    row_projection = project_boxes(
        filtered_box_list, "row",
        erosion=erodelevel)

    #
    y_comb = comb_from_projection(row_projection, rowThreshold, "row")
    y_comb.reverse()

    # columnThreshold = max(len(y_comb)*0.75,5)
    x_comb = comb_from_projection(column_projection, columnThreshold, "column")

    x_comb[0] = minx
    x_comb[-1] = maxx

    # Extend y_comb to page size if extend_y is true
    if extend_y:
        pageminy = min([box.bottom for box in box_list])
        pagemaxy = max([box.top for box in box_list])
        y_comb = comb_extend(y_comb, pageminy, pagemaxy)
        filtered_box_list = box_list

    # Applying the combs
    table_array = apply_combs(box_list, x_comb, y_comb)

    # Strip out leading and trailing spaces when atomise true
    if atomise:
        tmp_table = []
        for row in table_array:
            stripped_row = list(map(str.strip,row))
            tmp_table.append(stripped_row)
        table_array = tmp_table

    diagnostic_data = TableDiagnosticData(
        filtered_box_list,
        column_projection,
        row_projection,
        x_comb,
        y_comb)

    return table_array, diagnostic_data

def find_table_bounding_box(box_list, hints=[]):
    """ Returns one bounding box (minx, maxx, miny, maxy) for tables based
    on a boxlist
    """

    miny = min([box.bottom for box in box_list])
    maxy = max([box.top for box in box_list])
    minx = min([box.left for box in box_list])
    maxx = max([box.right for box in box_list])

    """ Get rid of LTChar for this stage """
    textLine_boxlist = box_list.filterByType('LTTextLineHorizontal')

    """ Try to reduce the y range with a threshold, wouldn't work for x"""
    yhisttop = textLine_boxlist.histogram(Leaf._top).rounder(2)
    yhistbottom = textLine_boxlist.histogram(Leaf._bottom).rounder(2)

    try:
        miny = min(threshold_above(yhistbottom, IS_TABLE_COLUMN_COUNT_THRESHOLD))
    # and the top of the top cell
        maxy = max(threshold_above(yhisttop, IS_TABLE_COLUMN_COUNT_THRESHOLD))
    except ValueError:
        # Value errors raised when min and/or max fed empty lists
        miny = None
        maxy = None
        #raise ValueError("table_threshold caught nothing")

    """The table miny and maxy can be modified by hints"""
    if hints:
        top_string = hints[0]  # "% Change"
        bottom_string = hints[1]  # "15.67%"
        hintedminy, hintedmaxy = get_min_and_max_y_from_hints(
            textLine_boxlist, top_string, bottom_string)
        if hintedminy:
            miny = hintedminy
        if hintedmaxy:
            maxy = hintedmaxy
    """Modify table minx and maxx with hints? """

    return (minx, maxx, miny, maxy)

def filter_box_list_by_position(box_list, minv, maxv, dir_fun):
    #TODO This should be in tree.py
    filtered_box_list = LeafList()
    # print minv, maxv, index
    for box in box_list:
        # box = boxstruct[0]
        if dir_fun(box) >= minv and dir_fun(box) <= maxv:
            # print box
            filtered_box_list.append(box)

    return filtered_box_list


def calculate_modal_height(box_list):
    height_list = []
    for box in box_list:
        if box.classname in ('LTTextLineHorizontal', 'LTChar'):
            height_list.append(fround(box.bbox[TOP] - box.bbox[BOTTOM]))

    modal_height = Counter(height_list).most_common(1)
    return modal_height[0][0]


def file_handle_from_url(URL):
    # TODO: move this function to a helper library
    response = requests.get(URL)
    fh = StringIO(response.content)
    return fh


if __name__ == '__main__':
    sys.stdout = codecs.getwriter('utf-8')(sys.stdout)
    if len(sys.argv) > 1:
        from .display import to_string
        with open(sys.argv[1], 'rb') as f:
            tables = get_tables(f)
            for i, table in enumerate(tables):
                print(("---- TABLE {} ----".format(i + 1)))
                print((to_string(table)))
    else:
        print(("Usage: {} <file.pdf>".format(sys.argv[0])))



#!/usr/bin/env python

from collections import defaultdict
from io import StringIO


def to_string(table):
    """
    Returns a list of the maximum width for each column across all rows
    >>> type(to_string([['foo', 'goodbye'], ['llama', 'bar']]))
    <type 'unicode'>
    """
    result = StringIO()

    (columns, rows) = get_dimensions(table)
        
    result.write("     {} columns, {} rows\n".format(columns, rows))
    col_widths = find_column_widths(table)
    table_width = sum(col_widths) + len(col_widths) + 2
    hbar = '    {}\n'.format('-' * table_width)

    result.write("      {}\n".format(' '.join(
        [str(col_index).rjust(width, ' ') for (col_index, width)
         in enumerate(col_widths)])))

    result.write(hbar)
    for row_index, row in enumerate(table):
        cells = [cell.rjust(width, ' ') for (cell, width)
                 in zip(row, col_widths)]
        result.write("{:>3} | {}|\n".format(row_index, '|'.join(cells)))
    result.write(hbar)
    result.seek(0)
    return str(result.read())


def get_dimensions(table):
    """
    Returns columns, rows for a table.
    >>> get_dimensions([['row1', 'apple', 'llama'], ['row2', 'plum', 'goat']])
    (3, 2)
    >>> get_dimensions([['row1', 'apple', 'llama'], ['row2', 'banana']])
    (3, 2)
    """
    rows = len(table)
    try:
        cols = max(len(row) for row in table)
    except ValueError:
        cols = 0
    return (cols, rows)


def find_column_widths(table):
    """
    Returns a list of the maximum width for each column across all rows
    >>> find_column_widths([['foo', 'goodbye'], ['llama', 'bar']])
    [5, 7]
    """
    col_widths = defaultdict(lambda: 0)
    for row_index, row in enumerate(table):
        for column_index, cell in enumerate(row):
            col_widths[column_index] = max(col_widths[column_index], len(cell))
    return [col_widths[col] for col in sorted(col_widths)]

if __name__ == '__main__':
    print((to_string([['foo', 'goodbye'], ['llama', 'bar']])))



#!/usr/bin/env python
# ScraperWiki Limited
# Ian Hopkinson, 2013-06-19
# -*- coding: utf-8 -*-


"""
tree classes which hold the parsed PDF document data
"""

import collections
from .counter import Counter

def _rounder(val,tol):
     """
     Utility function to round numbers to arbitrary tolerance
     """
     return float(round((1.0*val)/tol))*tol

class Histogram(Counter):
    def rounder(self, tol):
        c = Histogram()
        for item in self:
            c = c + Histogram({_rounder(item, tol): self[item]})
        return c

class Leaf(object):
    def __init__(self, obj):
        if isinstance(obj, tuple):
            (self.bbox, self.classname, self.text) = obj
        else:
            if obj.__class__.__name__ != 'LTAnon':
                self.bbox = obj.bbox
            else:
                self.bbox = (0, 0, 0, 0)
            self.classname = obj.__class__.__name__
            try:
                self.text = obj.get_text()
            except:
                self.text = ''

        (self.left, self.bottom, self.right, self.top) = self.bbox
        self.midline = (self.top+self.bottom)/2.0
        self.centreline = (self.left+self.right)/2.0
        self.width = self.right-self.left

    def __getitem__(self, i):
        """backwards-compatibility helper, don't use it!"""
        error = ("DEPRECATED: don't use leaf[x] - use these instead: "
                 "[0]: bbox, [1]: classname, [2]: text")
        raise RuntimeError(error)

        return [self.bbox, self.classname, self.text][i]

    def _left(self):
        return self.bbox[0]

    def _bottom(self):
        return self.bbox[1]

    def _right(self):
        return self.bbox[2]

    def _top(self):
        return self.bbox[3]

    def _midline(self):
        return self.midline

    def _centreline(self):
        return self.centreline

def children(obj):
    """get all descendants of nested iterables"""
    if isinstance(obj, collections.Iterable):
        for child in obj:
            for node in children(child):
                yield node
    yield obj

class LeafList(list):
    def purge_empty_text(self):
        return LeafList(box for box in self if box.text.strip()
                            or box.classname != 'LTTextLineHorizontal')

    def filterByType(self, flt=None):
        if not flt: return self
        return LeafList(box for box in self if box.classname in flt)

    def histogram(self, dir_fun):
        # index 0 = left, 1 = top, 2 = right, 3 = bottom
        for item in self:
            assert isinstance(item, Leaf), item
        return Histogram(dir_fun(box) for box in self)

    def populate(self, pdfpage, interested=['LTPage','LTTextLineHorizontal']):
    # def populate(self, pdfpage, interested=['LTPage','LTChar']):
        for obj in children(pdfpage):
            if not interested or obj.__class__.__name__ in interested:
                self.append(Leaf(obj))
        return self

    def count(self):
        return Counter(x.classname for x in self)



#!/usr/bin/env python
# ScraperWiki Limited
# Ian Hopkinson, 2013-06-17
# -*- coding: utf-8 -*-

"""
ContainsTables tests
"""

import sys
sys.path.append('code')

import pdftables

from nose.tools import assert_equals


def test_it_finds_no_tables_in_a_pdf_with_no_tables():
    fh = open('fixtures/sample_data/m27-dexpeg2-polymer.pdf', 'rb')
    assert_equals(
        [False, False, False, False, False, False, False, False],
        pdftables.contains_tables(fh))


def test_it_finds_tables_on_all_pages_AlmondBoard():
    fh = open('fixtures/sample_data/2012.01.PosRpt.pdf', 'rb')
    assert_equals(
        [True, True, True, True, True, True, True],
        pdftables.contains_tables(fh))


def test_it_finds_tables_on_some_pages_CONAB():
    fh = open('fixtures/sample_data/13_06_12_10_36_58_boletim_ingles_junho_2013.pdf', 'rb')
    TestList = [False]*32
    TestList[5:8] = [True]*3
    TestList[9:11] = [True]*2
    TestList[12] = True
    TestList[14] = True
    TestList[16:18] = [True]*2
    TestList[19:24] = [True]*5
    TestList[25:30] = [True]*5

    assert_equals(pdftables.contains_tables(fh), TestList)



#!/usr/bin/env python
# ScraperWiki Limited
# Ian Hopkinson, 2013-06-17
# -*- coding: utf-8 -*-

"""
comb tests
"""

import sys
sys.path.append('code')

from pdftables import (comb, comb_extend, 
                       comb_from_uppers_and_lowers,
                       find_minima)

from nose.tools import assert_equals, raises

def test_find_minima_works_in_simplest_case():
    projection = {5:2,6:1,7:2}
    assert_equals(6,find_minima(7, 5, projection=projection))
    
def test_find_minima_function_copes_with_multiple_values_at_minima():
    pass

def test_an_ascending_comb_is_extended_correctly():
    combarray = [2, 3, 4]
    assert_equals(
        [1.0, 2.0, 3.0, 4.0, 5.0, 6.0, 7.0, 8.0, 9.0, 10.0],
        comb_extend(combarray, 0.1, 10.5))


def test_a_descending_comb_is_extended_correctly():
    combarray = [5, 4, 3]
    assert_equals(comb_extend(combarray, 0.1, 8), [7, 6, 5, 4, 3, 2, 1])


def test_it_returns_minus_one_for_values_below_comb():
    combarray = [0, 1, 2, 3, 4, 5]
    assert_equals(comb(combarray, -1), -1)


def test_it_returns_minus_one_for_values_above_comb():
    combarray = [0, 1, 2, 3, 4, 5]
    assert_equals(comb(combarray, 6), -1)


def test_it_returns_correct_index_comb_ascending():
    combarray = [0, 1, 2, 3, 4, 5]
    assert_equals(comb(combarray, 0.5), 0)
    assert_equals(comb(combarray, 1.5), 1)
    assert_equals(comb(combarray, 4.5), 4)


def test_it_returns_correct_index_comb_descending():
    combarray = [5, 4, 3, 2, 1, 0]
    assert_equals(comb(combarray, 0.5), 4)
    assert_equals(comb(combarray, 1.5), 3)
    assert_equals(comb(combarray, 4.5), 0)


def test_comb_correctly_created_from_uppers_and_lowers():
    uppers = [100, 80, 60, 40, 20]
    lowers = [86, 66, 46, 26, 6]
    assert_equals(comb_from_uppers_and_lowers(uppers, lowers),
                  [100, 83, 63, 43, 23, 6])


@raises(Exception)
def test_raises_an_exception_for_an_unsorted_combarray():
    combarray = [5, 3, 4, 2, 1, 0]
    comb(combarray, 0.5)



#!/usr/bin/env python
# ScraperWiki Limited
# Ian Hopkinson, 2013-06-20
# -*- coding: utf-8 -*-

"""
tree Tests
"""

import sys
sys.path.append('pdftables')

import pdftables
#from tree import Leaf, LeafList
#import tree

from nose.tools import *

def test_basic_tree():
    leaf = pdftables.tree.Leaf(((11,22,33,44),"class","text"))
    assert_equals(leaf.bbox, (11,22,33,44))
    assert_equals((11, 22, 33, 44), leaf.bbox)
    assert_equals(22, leaf.bottom)
    assert_equals('class', leaf.classname)
    assert_equals('text', leaf.text)

def test_children():
    x = [1, [2, [[3, 4]]]]
    assert_equals(len(list(pdftables.tree.children(x))), 8)



#!/usr/bin/env python
# coding: utf-8

from __future__ import unicode_literals
from nose.tools import assert_equal
from os.path import join, dirname
import os

from pdftables import get_tables
from pdftables.display import to_string

SAMPLE_DIR = join(dirname(__file__), '..', 'fixtures', 'sample_data')
EXPECTED_DIR = join(dirname(__file__), '..', 'fixtures', 'expected_output')
ACTUAL_DIR = join(dirname(__file__), '..', 'fixtures', 'actual_output')


def _test_sample_data():
    for filename in os.listdir(SAMPLE_DIR):
        yield _test_sample_pdf, filename


def _test_sample_pdf(short_filename):
    with open(join(SAMPLE_DIR, short_filename),'rb') as f:
        tables = get_tables(f)

    assert_equal(get_expected_number_of_tables(short_filename), len(tables))
    for table_num, table in enumerate(tables):
        table_filename = "{}_{}.txt".format(short_filename, table_num)
        expected_filename = join(EXPECTED_DIR, table_filename)
        actual_filename = join(ACTUAL_DIR, table_filename)

        with open(actual_filename, 'w') as f:
            f.write(to_string(table).encode('utf-8'))

        diff_table_files(expected_filename, actual_filename)


def get_expected_number_of_tables(short_filename):
    result = len([fn for fn in os.listdir(EXPECTED_DIR)
                  if fn.startswith(short_filename)])
    if result == 0:
        print("NOTE: there is no 'expected' data for {0} in {1}: you probably "
              "want to review then copy files from {2}".format(
                  short_filename, EXPECTED_DIR, ACTUAL_DIR))
    return result


def diff_table_files(expected, result):
    with open(expected) as f:
        with open(result) as g:
            for line, (expected_line, actual_line) in enumerate(zip(f, g)):
                try:
                    assert_equal(expected_line, actual_line)
                except AssertionError:
                    print("{} and {} differ @ line {}".format(
                        expected, result, line + 1))
                    raise
